#include <Figments.h>
#include <Ringbuf.h>

#include <PubSubClient.h>
#include <ArduinoLog.h>

#include "../../Sequencer.h"

#ifdef BOARD_ESP8266
#include <ESP8266WiFi.h>
#elif defined(BOARD_ESP32)
#include <WiFi.h>
#endif


class MQTTTelemetry : public BufferedInputSource, OnlineTaskMixin {
  public:
    MQTTTelemetry();
    void setSequencer(Sequencer* seq) { m_sequencer = seq; }

    class LogPrinter : public Print {
      private:
        MQTTTelemetry* telemetry;
        Ringbuf<char, 512> buf;
      public:
        LogPrinter(MQTTTelemetry* telemetry) : telemetry(telemetry) {};
        size_t write(uint8_t byte) {
          char outBuf[512];
          if (byte == '\n') {
            size_t bufSize = buf.write(outBuf);
            outBuf[std::min(sizeof(outBuf), bufSize)] = 0;
            Serial.println(outBuf);
            String logTopic = telemetry->m_debugTopic + "/log";
            telemetry->m_mqtt.publish(logTopic.c_str(), outBuf);
          } else {
            buf.insert(byte);
          }
          return sizeof(byte);
        }
    };

    Print* logPrinter() {
        return &m_logPrinter;
    }

    void handleEventOnline(const InputEvent& evt) override;
    void loop() override;

    void loopOnline() override;

    void onOnline() override;
    void onOffline() override;

  private:
    String m_rootTopic;
    String m_debugTopic;

    void callback(char* topic, const char* payload);

    static void s_callback(char* topic, byte* payload, unsigned int length);
    char m_patternBuf[48];
    bool m_needHeartbeat = false;
    bool m_isOn = true;

    Sequencer *m_sequencer = 0;
    WiFiClient m_wifi;
    PubSubClient m_mqtt;
    LogPrinter m_logPrinter;
};
